/***************************************************************************
 *   Copyright (C) 2004 by Work@                                           *
 *   jstubbs@work-at.co.jp                                                 *
 *                                                                         *
 *   This program may be distributed under the terms of the Q Public       *
 *   License as defined by Trolltech AS of Norway and appearing in the     *
 *   file LICENSE.QPL included in the packaging of this file.              *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *
 ***************************************************************************/

#include "adder.h"

Adder::Adder(Cache* cache, bool display, QWidget* parent, const char* name)
	: QObject(parent, name)
{
	this->cache = cache;
	this->display = display;
	
	progress = new QProgressDialog(parent);
	progress->setMinimumDuration(1000);
	progress->setCaption(tr("Add"));
	
	connect(cache, SIGNAL(rssAvailable(QString, ItemList)),
	        this, SLOT(checkValid(QString, ItemList)));
	connect(cache, SIGNAL(rssInvalid(QString)),
	        this, SLOT(invalid(QString)));
	connect(cache, SIGNAL(urlInvalid(QString)),
	        this, SLOT(invalid(QString)));
	connect(cache, SIGNAL(urlModified(QString, QString)),
	        this, SLOT(modifyUrl(QString, QString)));
	if (display)
		connect(cache, SIGNAL(progress(QString, int)), this, SLOT(progressChange(QString, int)));
}

void
Adder::add(QString url)
{
	if (adding.contains(url))
		return;
	
	adding.append(url);
	if (adding.count() == 1) {
		cache->retrieve(url);
		progress->reset();
		progress->setLabelText(tr("Retrieving ") + url + tr(""));
	}
}

void
Adder::checkNext()
{
	if (adding.count() == 0)
		return;
	
	QString url = adding[0];
	cache->retrieve(url);
	progress->reset();
	progress->setLabelText(tr("Retrieving ") + url+tr(""));
}

void
Adder::checkValid(QString url, ItemList items)
{
	if (!adding.contains(url))
		return;
	
	progress->setProgress(100, 100);
	adding.remove(url);
	
	Site site;
	site.url = url;
	site.enabled = true;
	site.title = items[0].channel.title;
	
	emit validSite(site);
	
	checkNext();
}

void
Adder::invalid(QString url)
{
	if (!adding.contains(url))
		return;
	progress->setProgress(100, 100);
	
	adding.remove(url);
	
	emit invalidUrl(url);
	
	checkNext();
}

void
Adder::modifyUrl(QString prev, QString cur)
{
	if (!adding.contains(prev))
		return;
	adding.remove(prev);
	adding.push_front(cur);
}

void
Adder::progressChange(QString url, int pcnt)
{
	if (url != adding[0])
		return;
	
	if (pcnt == 0)
		return;
	progress->setProgress(pcnt, 100);
}
