/***************************************************************************
 *   Copyright (C) 2004 by Work@                                           *
 *   jstubbs@work-at.co.jp                                                 *
 *                                                                         *
 *   This program may be distributed under the terms of the Q Public       *
 *   License as defined by Trolltech AS of Norway and appearing in the     *
 *   file LICENSE.QPL included in the packaging of this file.              *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *
 ***************************************************************************/

#ifndef __CACHE_H__
#define __CACHE_H__

#include <qobject.h>
#include <qmemarray.h>
#include <qdict.h>
#include <qmap.h>
#include <qdatetime.h>
#include <qstring.h>
#include <qtimer.h>
#include <qhttp.h>

#include "datatypes.h"

class Cache
	: public QObject
{
	Q_OBJECT
	
	signals:
		void cacheUpdating();
		void cacheCurrent();
		void rssAvailable(QString url, ItemList items);
		void rssInvalid(QString url);
		void urlInvalid(QString url);
		void urlModified(QString, QString);
		void progress(QString url, int pcnt);
		void rssTitle(QString url, QString title);
	
	public slots:
		void invalidate();
		void setTimeout(int timeout);
		void retrieve(QString url, bool check = true);
	
	protected slots:
		void progress(int, int);
		void receiveResponse(const QHttpResponseHeader& header);
		void checkData(const QHttpResponseHeader&);
		void complete(int, bool);
		void fetchNext(bool someval=false);
		void http_timeout();
	
	public:
		Cache(QObject* parent = 0, const char* name = 0);
	
	private:
		void parse(QString, QString*, int*, QString*, QString*);

		struct CacheEntry
		{
			ItemList items;
			uint time;
			QString lastmodified;
		};
		
		QMap<QString, CacheEntry> entries;
		QMap<QString, QString> mapping;
		QHttp* http;
		QStringList requests;
		int currentid;
		QHttpResponseHeader response;
		int timeout;
		QByteArray data;
		QTimer timer;
		QString lasthost;
		bool retry;
};

#endif
