/***************************************************************************
 *   Copyright (C) 2004 by Work@                                           *
 *   jstubbs@work-at.co.jp                                                 *
 *                                                                         *
 *   This program may be distributed under the terms of the Q Public       *
 *   License as defined by Trolltech AS of Norway and appearing in the     *
 *   file LICENSE.QPL included in the packaging of this file.              *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *
 ***************************************************************************/

#ifndef __MAINWINDOW_H__
#define __MAINWINDOW_H__

#include <qwidget.h>
#include <qpopupmenu.h>
#include <qpoint.h>
#include <qclipboard.h>
#include <qprocess.h>

#include "settings.h"
#include "cache.h"
#include "configdialog.h"
#include "manager.h"
#include "ticker.h"
#include "skin.h"
#include "tabulator.h"

class MinimizeHelper
	: public QWidget
{
	Q_OBJECT
	
	signals:
		void aboutToShow();
	
	public slots:
		void hide()
		{
			QWidget::hide();
			minimized = false;
		}
		
		void showMinimized()
		{
			QWidget::showMinimized();
			minimized = true;
		}
		
	public:
		MinimizeHelper(QWidget* parent = 0, const char* name = 0)
			: QWidget(parent, name, Qt::WStyle_NoBorder)
		{
			minimized = false;
			QBitmap mask(100, 100);
			mask.fill(Qt::color0);
			setMask(mask);
		}
	
	protected:
		bool event(QEvent* e)
		{
			if (minimized && e->type() == QEvent::WindowStateChange) {
				minimized = false;
				emit aboutToShow();
			} else if (!minimized && e->type() == QEvent::ShowMinimized) {
				minimized = true;
			}
			if (minimized)
				return QWidget::event(e);
			else
				return true;
		}
	
	private:
		bool minimized;
};

class MainWindow
 : public QWidget
{
	Q_OBJECT
	
	signals:
		void urlClosed(QString);
	
	protected slots:
		void verInfo();
		void setSkin(const QString&);
		void openUrl(QString);
		void display(Item);
		void wait();
		void error();
		void setTimeout(int);
		void reload();
		void beginReload();
		void endReload();
		void newSites(SiteList);
		void headlineTimeout();
		void clipboardChanged();
		void quit();
		void opmlimport();
		void opmlexport();
		void jumpToIndex(int);
		void fillJumpMenu();
		void cleanProcesses(bool force = false);
	
	public:
		MainWindow(QWidget* parent = 0, const char* name = 0);
	
	protected:
		void mousePressEvent(QMouseEvent* e);
		void mouseMoveEvent(QMouseEvent* e);
		void mouseReleaseEvent(QMouseEvent* e);
		void paintEvent(QPaintEvent* e);
		void showEvent(QShowEvent*);
		void wheelEvent(QWheelEvent *e);
		void contextMenuEvent(QContextMenuEvent* e);
		void keyPressEvent(QKeyEvent*);
		void scroll();
	
	private:
		MinimizeHelper* helper;
		Settings* settings;
		Cache* cache;
		Tabulator* tabulator;
		Adder* adder;
		ConfigDialog* confdialog;
		Manager* manager;
		Ticker* ticker;
		Skin* skin;
		QPopupMenu* popup;
		QPopupMenu* jumpmenu;
		QClipboard *clipboard;
		QTimer timer;
		int timeout;
		bool reloading;
		bool pausing;
		bool waiting;
		bool left;
		QPoint mousepos;
		bool moving;
		bool moved;
		bool wheeling;
		QDict<QProcess> processes;
};

#endif
