/***************************************************************************
 *   Copyright (C) 2004 by Work@                                           *
 *   jstubbs@work-at.co.jp                                                 *
 *                                                                         *
 *   This program may be distributed under the terms of the Q Public       *
 *   License as defined by Trolltech AS of Norway and appearing in the     *
 *   file LICENSE.QPL included in the packaging of this file.              *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *
 ***************************************************************************/

#include <qmap.h>
#include <qfile.h>
#include <qimage.h>
#include <qpainter.h>
#include <qdir.h>

#include "settings.h"
#include "skin.h"
#include "paths.h"

Skin::Skin(const QString& name)
{
	QString path = USER;
	if (!QDir(USER + name).exists())
		path = SHARED;
	path += name + "/";
	_reloadoff = QPixmap(path + "btn1-1.bmp");
	_reloadon = QPixmap(path + "btn1-2.bmp");
	_pauseoff = QPixmap(path + "btn2-1.bmp");
	_pauseon = QPixmap(path + "btn2-2.bmp");
	_base = QPixmap(path + "base.bmp");
	
	QImage image = QPixmap(path + "base.bmp", "BMP", Qt::ThresholdDither | Qt::AvoidDither).convertToImage();
	_mask = QBitmap(image.width(), image.height(), TRUE);
	QPainter p(&_mask);
	QRgb maskcolor = image.pixel(0, image.height() - 1);
	p.setBrush(Qt::color1);
	for (int y = 0; y < image.height(); y++) {
		for (int x = 0; x < image.width(); x++) {
			if (image.pixel(x, y) != maskcolor) {
				p.drawPoint(x, y);
			}
		}
	}
	p.end();
	
	ConfigFile f(path+"/skin.conf");
	f.read();
	_size = getRect(&f, "base");
	_close = getRect(&f, "close");
	_minimize = getRect(&f, "minibtn");
	_reload = getRect(&f, "reload");
	_pause = getRect(&f, "stop");
	_ticker = getRect(&f, "news");
	_tabulate = getRect(&f, "list");
}

QRect
Skin::getRect(const ConfigFile* f, const QString& name)
{
	int x1 = f->get(name+"x1", (int)0);
	int x2 = f->get(name+"x2", (int)0);
	int y1 = f->get(name+"y1", (int)0);
	int y2 = f->get(name+"y2", (int)0);
	return QRect(x1,y1,x2-x1,y2-y1);
}

QStringList
Skin::available()
{
	QStringList skins;
	
	QStringList userskins = QDir(USER).entryList(QDir::Dirs);
	for (int i = 0; i < userskins.count(); i++) {
		QString skin = userskins[i];
		QStringList contents = QDir(USER + skin).entryList();
		if (contents.contains("base.bmp") && contents.contains("skin.conf") &&
		    contents.contains("btn1-1.bmp") && contents.contains("btn1-2.bmp") &&
		    contents.contains("btn2-1.bmp") && contents.contains("btn2-2.bmp"))
			skins.append(skin);
	}
	
	QStringList sharedskins = QDir(SHARED).entryList(QDir::Dirs);
	for (int i = 0; i < sharedskins.count(); i++) {
		QString skin = sharedskins[i];
		if (skins.findIndex(skin) != -1)
			continue;
		QStringList contents = QDir(SHARED + skin).entryList();
		if (contents.contains("base.bmp") && contents.contains("skin.conf") &&
		    contents.contains("btn1-1.bmp") && contents.contains("btn1-2.bmp") &&
		    contents.contains("btn2-1.bmp") && contents.contains("btn2-2.bmp"))
			skins.append(skin);
	}
	return skins;
}
