/***************************************************************************
 *   Copyright (C) 2004 by Work@                                           *
 *   jstubbs@work-at.co.jp                                                 *
 *                                                                         *
 *   This program may be distributed under the terms of the Q Public       *
 *   License as defined by Trolltech AS of Norway and appearing in the     *
 *   file LICENSE.QPL included in the packaging of this file.              *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *
 ***************************************************************************/


#include <stdlib.h>
#include <qfile.h>
#include <qdir.h>

#include "tabulator.h"
#include "paths.h"

Tabulator::Tabulator(Settings* settings, Cache* cache, QWidget* parent, const char* name)
	: QObject(parent, name)
{
	this->settings = settings;
	this->cache = cache;
	this->progress = new QProgressDialog(parent);
	this->progress->setCaption(tr("Creating..."));
	this->progress->setMinimumDuration(200);
	connect(cache, SIGNAL(rssAvailable(QString, ItemList)),
	        this, SLOT(receive(QString, ItemList)));
	connect(cache, SIGNAL(urlInvalid(QString)), this, SLOT(skip(QString)));
	connect(cache, SIGNAL(rssInvalid(QString)), this, SLOT(skip(QString)));
	connect(progress, SIGNAL(cancelled()), this, SLOT(cancel()));
	
	fidx = -1;
}

void
Tabulator::create()
{
	if (fidx != -1)
		return;
	
	sites = settings->sites();
	
	progress->setTotalSteps(sites.count());
	
	QCString fn = TEMP.local8Bit()+"rabbitticker_XXXXXX";
	fidx = mkstemp(fn.data());
	
	if (fidx == -1)
		return;
	
	QFile f;
	f.open(IO_WriteOnly, fidx);
	files.append(QString(fn));
	
	QFile tmpl(USER+"rthead.txt");
	bool success = tmpl.open(IO_ReadOnly);
	if (!success) {
		tmpl.setName(SHARED+"rthead.txt");
		tmpl.open(IO_ReadOnly);
	}
	QByteArray data = tmpl.readAll();
	f.writeBlock(data);
	tmpl.close();
	f.close();
	
	next();
}

void
Tabulator::next()
{
	while (sites.count() != 0) {
		if (sites[0].enabled) {
			cache->retrieve(sites[0].url);
			progress->setLabelText(sites[0].title);
			return;
		} else {
			sites.pop_front();
			progress->setProgress(progress->progress() + 1);
		}
	}
	done();
}

void
Tabulator::skip(QString url)
{
	if (fidx == -1)
		return;
	
	if (sites.count() == 0)
		return;
	
	if (sites[0].url == url) {
		sites.pop_front();
		progress->setProgress(progress->progress() + 1);
		next();
	}
}

void
Tabulator::receive(QString url, ItemList items)
{
	if (fidx == -1)
		return;
	
	if (sites.count() == 0)
		return;
	
	if (sites[0].url != url)
		return;
	
	QFile f;
	f.open(IO_WriteOnly, fidx);
	QTextStream ts(&f);
	ts.setEncoding(QTextStream::UnicodeUTF8);
	
	ts << "<div class=\"newsbody\"><h3 class=\"title\"><a href=\"";
	ts << items[0].channel.url << "\" ";
	if (items[0].channel.comment != "") {
		QString title = items[0].channel.comment;
		title.replace("\"", "'");
		ts << "title=\"" << title << "\" ";
	}
	ts << ">" << items[0].channel.title << "</a></h3>\n";
	
	for (unsigned int i = 0; i < items.count(); i++) {
		ts << "<a href=\"" << items[i].headline.url << "\" ";
		if (items[0].headline.comment != "") {
			QString title = items[i].headline.comment;
			title.replace("\"", "'");
			ts << "title=\"" << title << "\" ";
		}
		ts << ">" << items[i].headline.title << "</a><br />\n";
	}
	ts << "<br /></div><br />\n";
	f.close();
	sites.pop_front();
	progress->setProgress(progress->progress() + 1);
	
	next();
}

void
Tabulator::done()
{
	QFile f;
	f.open(IO_WriteOnly, fidx);
	QFile tmpl(USER+"rtfoot.txt");
	bool success = tmpl.open(IO_ReadOnly);
	if (!success) {
		tmpl.setName(SHARED+"rtfoot.txt");
		tmpl.open(IO_ReadOnly);
	}
	QByteArray data = tmpl.readAll();
	f.writeBlock(data);
	tmpl.close();
	f.close();
	close(fidx);
	fidx = -1;
	emit fileReady(files.last());
	progress->setProgress(progress->progress() + 1);
}

void
Tabulator::clean(QString fn)
{
	if (files.contains(fn)) {
		QDir d;
		d.remove(fn, true);
	}
}

void
Tabulator::cancel()
{
	close(fidx);
	fidx = -1;
	clean(files.last());
}
